package com.qmth.teachcloud.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.dto.CourseInfoDto;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.mapper.BasicCourseMapper;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 课程表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class BasicCourseServiceImpl extends ServiceImpl<BasicCourseMapper, BasicCourse> implements BasicCourseService {

    @Resource
    private BasicCourseMapper basicCourseMapper;



    @Override
    public List<BasicCourse> listCoursesByUserId(Long userId) {
        return this.baseMapper.listCoursesByUserId(userId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void verifyCourseInfo(Long schoolId, String courseCode, String courseName, Long userId) {
        QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicCourse::getSchoolId, schoolId).eq(BasicCourse::getCode, courseCode);

        BasicCourse tmp = this.getOne(queryWrapper);
        if (tmp != null) {
            if (!courseName.equals(tmp.getName())) {
                throw ExceptionResultEnum.ERROR
                        .exception("课程代码和课程名称不匹配,课程代码为 ：'" + courseCode + "',输入的课程名称 ：'" + courseName + "',实际该代码对应的课程名称 ：'" + tmp.getName() + "'。");
            }
        } else {
            BasicCourse basicCourse = new BasicCourse();
            basicCourse.setName(courseName);
            basicCourse.setCode(courseCode);
            basicCourse.setSchoolId(schoolId);
            basicCourse.setId(SystemConstant.getDbUuid());
            basicCourse.setCreateId(userId);
            basicCourse.setCreateTime(System.currentTimeMillis());
            this.saveOrUpdate(basicCourse);
        }
    }

    @Override
    public List<CourseInfoDto> findByUserLoginNameAndRealName(String loginName, String realName) {
        System.out.println("---");
        if (loginName == null) {
            loginName = "";
        }
        if (realName == null) {
            realName = "";
        }
        return basicCourseMapper.findByUserLoginNameAndRealName(loginName, realName);
    }

    @Override
    public BasicCourse findByCourseCode(String courseCode,Long schoolId) {
        return this.getOne(new QueryWrapper<BasicCourse>().lambda()
                .eq(BasicCourse::getCode,courseCode)
                .eq(BasicCourse::getSchoolId,schoolId));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void createCourse(Long schoolId, String courseCode, String courseName) {
        // 向'basic_course'表新增数据
        List<BasicCourse> basicCourseList = this.list(new QueryWrapper<BasicCourse>().lambda()
                .eq(BasicCourse::getSchoolId, schoolId).eq(BasicCourse::getCode, courseCode));
        if (basicCourseList.size() > 1) {
            throw ExceptionResultEnum.ERROR.exception("学校id为【" + schoolId + "】,课程编号为【" + courseCode + "】的课程在 'basic_course' 表中存在多条异常");
        }
        // 课程
        Long id;
        if (basicCourseList.size() > 0){
            id = basicCourseList.get(0).getId();
        }else {
            id = SystemConstant.getDbUuid();
        }
        BasicCourse basicCourse = new BasicCourse();
        basicCourse.setId(id);
        basicCourse.setCode(courseCode);
        basicCourse.setName(courseName);
        basicCourse.setSchoolId(schoolId);
        this.saveOrUpdate(basicCourse);
    }
}
